@@ -23,22 +23,21 @@
 #include <stdlib.h>
 #include <errno.h>
 
+/* prevent libcdio from redefining PACKAGE, VERSION, etc. */
+#define EXTERNAL_LIBCDIO_CONFIG_H
+
 #include <cdio/cdio.h>
 #include <cdio/cdtext.h>
 #include <cdio/track.h>
-#include <cdio/cdda.h>
 #include <cdio/audio.h>
 #include <cdio/sector.h>
 #include <cdio/cd_types.h>
 
-/* libcdio's header files #define these */
-#undef PACKAGE
-#undef PACKAGE_BUGREPORT
-#undef PACKAGE_NAME
-#undef PACKAGE_STRING
-#undef PACKAGE_TARNAME
-#undef PACKAGE_VERSION
-#undef VERSION
+#if LIBCDIO_VERSION_NUM >= 90
+#include <cdio/paranoia/cdda.h>
+#else
+#include <cdio/cdda.h>
+#endif
 
 #include <cddb/cddb.h>
 
@@ -54,8 +53,6 @@
 #include <libaudgui/libaudgui.h>
 #include <libaudgui/libaudgui-gtk.h>
 
-#include "config.h"
-
 #define DEF_STRING_LEN 256
 
 #define MIN_DISC_SPEED 2
@@ -269,9 +266,9 @@ static void cdaudio_set_strinfo (trackinfo_t * t,
                                  const char * performer, const char * name,
                                  const char * genre)
 {
-    g_strlcpy (t->performer, performer, DEF_STRING_LEN);
-    g_strlcpy (t->name, name, DEF_STRING_LEN);
-    g_strlcpy (t->genre, genre, DEF_STRING_LEN);
+    g_strlcpy (t->performer, performer ? performer : "", DEF_STRING_LEN);
+    g_strlcpy (t->name, name ? name : "", DEF_STRING_LEN);
+    g_strlcpy (t->genre, genre ? genre : "", DEF_STRING_LEN);
 }
 
 /* thread safe (mutex may be locked) */
@@ -622,23 +619,32 @@ static void scan_cd (void)
     }
 
     /* get trackinfo[0] cdtext information (the disc) */
+    cdtext_t *pcdtext = NULL;
     if (aud_get_bool ("CDDA", "use_cdtext"))
     {
         AUDDBG ("getting cd-text information for disc\n");
-        cdtext_t *pcdtext = cdio_get_cdtext (pcdrom_drive->p_cdio, 0);
+#if LIBCDIO_VERSION_NUM >= 90
+        pcdtext = cdio_get_cdtext (pcdrom_drive->p_cdio);
+        if (pcdtext == NULL)
+#else
+        pcdtext = cdio_get_cdtext (pcdrom_drive->p_cdio, 0);
         if (pcdtext == NULL || pcdtext->field[CDTEXT_TITLE] == NULL)
+#endif
         {
             AUDDBG ("no cd-text available for disc\n");
         }
         else
         {
             cdaudio_set_strinfo (&trackinfo[0],
-                                 pcdtext->field[CDTEXT_PERFORMER] ? pcdtext->
-                                 field[CDTEXT_PERFORMER] : "",
-                                 pcdtext->field[CDTEXT_TITLE] ? pcdtext->
-                                 field[CDTEXT_TITLE] : "",
-                                 pcdtext->field[CDTEXT_GENRE] ? pcdtext->
-                                 field[CDTEXT_GENRE] : "");
+#if LIBCDIO_VERSION_NUM >= 90
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_PERFORMER, 0),
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_TITLE, 0),
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_GENRE, 0));
+#else
+                                 pcdtext->field[CDTEXT_PERFORMER],
+                                 pcdtext->field[CDTEXT_TITLE],
+                                 pcdtext->field[CDTEXT_GENRE]);
+#endif
         }
     }
 
@@ -646,7 +652,7 @@ static void scan_cd (void)
     bool_t cdtext_was_available = FALSE;
     for (trackno = firsttrackno; trackno <= lasttrackno; trackno++)
     {
-        cdtext_t *pcdtext = NULL;
+#if LIBCDIO_VERSION_NUM < 90
         if (aud_get_bool ("CDDA", "use_cdtext"))
         {
             AUDDBG ("getting cd-text information for track %d\n", trackno);
@@ -657,16 +663,20 @@ static void scan_cd (void)
                 pcdtext = NULL;
             }
         }
+#endif
 
         if (pcdtext != NULL)
         {
             cdaudio_set_strinfo (&trackinfo[trackno],
-                                 pcdtext->field[CDTEXT_PERFORMER] ? pcdtext->
-                                 field[CDTEXT_PERFORMER] : "",
-                                 pcdtext->field[CDTEXT_TITLE] ? pcdtext->
-                                 field[CDTEXT_TITLE] : "",
-                                 pcdtext->field[CDTEXT_GENRE] ? pcdtext->
-                                 field[CDTEXT_GENRE] : "");
+#if LIBCDIO_VERSION_NUM >= 90
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_PERFORMER, trackno),
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_TITLE, trackno),
+                                 cdtext_get(pcdtext, CDTEXT_FIELD_GENRE, trackno));
+#else
+                                 pcdtext->field[CDTEXT_PERFORMER],
+                                 pcdtext->field[CDTEXT_TITLE],
+                                 pcdtext->field[CDTEXT_GENRE]);
+#endif
             cdtext_was_available = TRUE;
         }
         else
