@@ -30,7 +30,6 @@
 #undef FFAUDIO_DOUBLECHECK  /* Doublecheck probing result for debugging purposes */
 #undef FFAUDIO_NO_BLACKLIST /* Don't blacklist any recognized codecs/formats */
 
-#include "config.h"
 #include "ffaudio-stdinc.h"
 #include <audacious/i18n.h>
 #include <audacious/debug.h>
@@ -55,7 +54,7 @@ static gint lockmgr (void * * mutexp, enum AVLockOp op)
     switch (op)
     {
     case AV_LOCK_CREATE:
-        * mutexp = g_slice_new (pthread_t);
+        * mutexp = g_slice_new (pthread_mutex_t);
         pthread_mutex_init (* mutexp, NULL);
         break;
     case AV_LOCK_OBTAIN:
@@ -66,7 +65,7 @@ static gint lockmgr (void * * mutexp, enum AVLockOp op)
         break;
     case AV_LOCK_DESTROY:
         pthread_mutex_destroy (* mutexp);
-        g_slice_free (pthread_t, * mutexp);
+        g_slice_free (pthread_mutex_t, * mutexp);
         break;
     }
 
@@ -413,9 +412,13 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
     gint i, stream_id, errcount;
     gboolean codec_opened = FALSE;
     gint out_fmt;
+    gboolean planar;
     gboolean seekable;
     gboolean error = FALSE;
 
+    void *buf = NULL;
+    gint bufsize = 0;
+
     AVFormatContext * ic = open_input_file (filename, file);
     if (! ic)
         return FALSE;
@@ -447,11 +450,18 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
 
     codec_opened = TRUE;
 
-    switch (c->sample_fmt) {
-        case AV_SAMPLE_FMT_U8: out_fmt = FMT_U8; break;
-        case AV_SAMPLE_FMT_S16: out_fmt = FMT_S16_NE; break;
-        case AV_SAMPLE_FMT_S32: out_fmt = FMT_S32_NE; break;
-        case AV_SAMPLE_FMT_FLT: out_fmt = FMT_FLOAT; break;
+    switch (c->sample_fmt)
+    {
+        case AV_SAMPLE_FMT_U8: out_fmt = FMT_U8; planar = FALSE; break;
+        case AV_SAMPLE_FMT_S16: out_fmt = FMT_S16_NE; planar = FALSE; break;
+        case AV_SAMPLE_FMT_S32: out_fmt = FMT_S32_NE; planar = FALSE; break;
+        case AV_SAMPLE_FMT_FLT: out_fmt = FMT_FLOAT; planar = FALSE; break;
+
+        case AV_SAMPLE_FMT_U8P: out_fmt = FMT_U8; planar = TRUE; break;
+        case AV_SAMPLE_FMT_S16P: out_fmt = FMT_S16_NE; planar = TRUE; break;
+        case AV_SAMPLE_FMT_S32P: out_fmt = FMT_S32_NE; planar = TRUE; break;
+        case AV_SAMPLE_FMT_FLTP: out_fmt = FMT_FLOAT; planar = TRUE; break;
+
     default:
         fprintf (stderr, "ffaudio: Unsupported audio format %d\n", (int) c->sample_fmt);
         goto error_exit;
@@ -567,8 +577,23 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
             if (! decoded)
                 continue;
 
-            playback->output->write_audio (frame->data[0], FMT_SIZEOF (out_fmt)
-             * c->channels * frame->nb_samples);
+            gint size = FMT_SIZEOF (out_fmt) * c->channels * frame->nb_samples;
+
+            if (planar)
+            {
+                if (bufsize < size)
+                {
+                    buf = realloc (buf, size);
+                    bufsize = size;
+                }
+
+                audio_interlace ((const void * *) frame->data, out_fmt,
+                 c->channels, buf, frame->nb_samples);
+                playback->output->write_audio (buf, size);
+            }
+            else
+                playback->output->write_audio (frame->data[0], size);
+
             av_free (frame);
         }
 
@@ -577,9 +602,6 @@ static gboolean ffaudio_play (InputPlayback * playback, const gchar * filename,
     }
 
 error_exit:
-
-    AUDDBG("decode loop finished, shutting down\n");
-
     stop_flag = TRUE;
 
     if (pkt.data)
@@ -589,7 +611,8 @@ error_exit:
     if (ic != NULL)
         close_input_file(ic);
 
-    AUDDBG("exiting thread\n");
+    free (buf);
+
     return ! error;
 }
 
@@ -691,7 +714,6 @@ AUD_INPUT_PLUGIN
     .extensions = ffaudio_fmts,
     .update_song_tuple = ffaudio_write_tag,
 
-    /* FFMPEG probing takes forever on network files, so try everything else
-     * first. -jlindgren */
+    /* lowest priority fallback */
     .priority = 10,
 )
