@@ -18,14 +18,6 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-/*
- * TODO:
- *   - do the VFS operations in another thread
- *   - why don't we have a url encoding function?
- *     ... or do we and I'm just smoking crack?
- *   - no idea what this code does if lyricwiki is down - probably crashes!
- */
-
 #include <stdio.h>
 #include <glib.h>
 #include <string.h>
@@ -45,22 +37,25 @@
 #include <libaudcore/hook.h>
 #include <libaudcore/vfs_async.h>
 
-#include "config.h"
-#include "urlencode.h"
+/* all strings in this struct are pooled */
+typedef struct {
+	char *filename; /* of song file */
+	char *title, *artist;
+	char *uri; /* URI we are trying to retrieve */
+} LyricsState;
+
+static LyricsState state;
 
 /*
  * Suppress libxml warnings, because lyricwiki does not generate anything near
  * valid HTML.
  */
-void
-libxml_error_handler(void *ctx, const char *msg, ...)
+static void libxml_error_handler(void *ctx, const char *msg, ...)
 {
-
 }
 
-/* Oh, this is going to be fun... */
-gchar *
-scrape_lyrics_from_lyricwiki_edit_page(const gchar *buf, gsize len)
+/* free() returned text */
+static char *scrape_lyrics_from_lyricwiki_edit_page(const char *buf, int64_t len)
 {
 	xmlDocPtr doc;
 	gchar *ret = NULL;
@@ -112,14 +107,14 @@ give_up:
 				GMatchInfo *match_info;
 				GRegex *reg;
 
-				reg = g_regex_new("<(lyrics?)>(.*)</\\1>", (G_REGEX_MULTILINE | G_REGEX_DOTALL | G_REGEX_UNGREEDY), 0, NULL);
+				reg = g_regex_new("<(lyrics?)>[[:space:]]*(.*?)[[:space:]]*</\\1>", (G_REGEX_MULTILINE | G_REGEX_DOTALL), 0, NULL);
 				g_regex_match(reg, (gchar *) lyric, G_REGEX_MATCH_NEWLINE_ANY, &match_info);
 
 				ret = g_match_info_fetch(match_info, 2);
-				if (!g_utf8_collate(ret, "\n<!-- PUT LYRICS HERE (and delete this entire line) -->\n"))
+				if (!g_utf8_collate(ret, "<!-- PUT LYRICS HERE (and delete this entire line) -->"))
 				{
-					g_free(ret);
-					ret = NULL;
+					free(ret);
+					ret = strdup(_("No lyrics available"));
 				}
 
 				g_regex_unref(reg);
@@ -134,12 +129,22 @@ give_up:
 	return ret;
 }
 
-gchar *
-scrape_uri_from_lyricwiki_search_result(const gchar *buf, gsize len)
+/* str_unref() returned string */
+static char *scrape_uri_from_lyricwiki_search_result(const char *buf, int64_t len)
 {
 	xmlDocPtr doc;
 	gchar *uri = NULL;
 
+	/*
+	 * workaround buggy lyricwiki search output where it cuts the lyrics
+	 * halfway through the UTF-8 symbol resulting in invalid XML.
+	 */
+	GRegex *reg;
+
+	reg = g_regex_new("<(lyrics?)>.*</\\1>", (G_REGEX_MULTILINE | G_REGEX_DOTALL | G_REGEX_UNGREEDY), 0, NULL);
+	gchar *newbuf = g_regex_replace_literal(reg, buf, len, 0, "", G_REGEX_MATCH_NEWLINE_ANY, NULL);
+	g_regex_unref(reg);
+
 	/*
 	 * temporarily set our error-handling functor to our suppression function,
 	 * but we have to set it back because other components of Audacious depend
@@ -150,7 +155,7 @@ scrape_uri_from_lyricwiki_search_result(const gchar *buf, gsize len)
 	 * parsing and hope for the best.
 	 */
 	xmlSetGenericErrorFunc(NULL, libxml_error_handler);
-	doc = xmlParseMemory(buf, (int) len);
+	doc = xmlParseMemory(newbuf, strlen(newbuf));
 	xmlSetGenericErrorFunc(NULL, NULL);
 
 	if (doc != NULL)
@@ -169,7 +174,9 @@ scrape_uri_from_lyricwiki_search_result(const gchar *buf, gsize len)
 				lyric = xmlNodeGetContent(cur);
 				basename = g_path_get_basename((gchar *) lyric);
 
-				uri = g_strdup_printf("http://lyrics.wikia.com/index.php?action=edit&title=%s", basename);
+				uri = str_printf("http://lyrics.wikia.com/index.php?action=edit"
+				 "&title=%s", basename);
+
 				g_free(basename);
 				xmlFree(lyric);
 			}
@@ -178,141 +185,111 @@ scrape_uri_from_lyricwiki_search_result(const gchar *buf, gsize len)
 		xmlFreeDoc(doc);
 	}
 
+	g_free(newbuf);
+
 	return uri;
 }
 
-gboolean
-check_current_track(Tuple *tu)
-{
-	gboolean ret = TRUE;
-	gint playlist, pos, i;
-	gint fields[] = {FIELD_ARTIST, FIELD_TITLE};
-	Tuple *cu = NULL;
+static void update_lyrics_window(const char *title, const char *artist, const char *lyrics);
 
-	if (tu == NULL)
+static bool_t get_lyrics_step_3(void *buf, int64_t len, void *requri)
+{
+	if (strcmp(state.uri, requri))
+	{
+		free(buf);
+		str_unref(requri);
 		return FALSE;
+	}
+	str_unref(requri);
 
-	playlist = aud_playlist_get_playing();
-	pos = aud_playlist_get_position(playlist);
-	cu = aud_playlist_entry_get_tuple(playlist, pos, FALSE);
-
-	if (cu == NULL)
+	if(!len)
+	{
+		SPRINTF(error, _("Unable to fetch %s"), state.uri);
+		update_lyrics_window(_("Error"), NULL, error);
+		free(buf);
 		return FALSE;
-
-	for (i = 0; i < sizeof(fields)/sizeof(gint); i++)
-	{
-		gchar * string1 = tuple_get_str (tu, fields[i], NULL);
-		gchar * string2 = tuple_get_str (cu, fields[i], NULL);
-
-		if (string1 == NULL && string2 == NULL)
-			continue;
-
-		if (string1 == NULL || string2 == NULL ||
-			strcmp(string1, string2) != 0)
-		{
-			ret = FALSE;
-			str_unref (string1);
-			str_unref (string2);
-			break;
-		}
-
-		str_unref (string1);
-		str_unref (string2);
 	}
 
-	tuple_unref(cu);
-	return ret;
-}
+	char *lyrics = scrape_lyrics_from_lyricwiki_edit_page(buf, len);
 
-void update_lyrics_window(const Tuple *tu, const gchar *lyrics);
-
-gboolean
-get_lyrics_step_3(gchar *buf, gint64 len, Tuple *tu)
-{
-	gchar *lyrics = NULL;
-
-	if (buf != NULL)
+	if(!lyrics)
 	{
-		lyrics = scrape_lyrics_from_lyricwiki_edit_page(buf, len);
-		g_free(buf);
+		SPRINTF(error, _("Unable to parse %s"), state.uri);
+		update_lyrics_window(_("Error"), NULL, error);
+		free(buf);
+		return FALSE;
 	}
 
-	if (check_current_track(tu))
-		update_lyrics_window(tu, lyrics);
-
-	tuple_unref (tu);
-
-	if (lyrics != NULL)
-		g_free(lyrics);
-
-	return buf == NULL ? FALSE : TRUE;
-}
-
-gboolean
-get_lyrics_step_2(gchar *buf, gint64 len, Tuple *tu)
-{
-	gchar *uri;
-
-	uri = scrape_uri_from_lyricwiki_search_result(buf, len);
-	if (uri == NULL)
-	{
-		if (check_current_track(tu))
-			update_lyrics_window(tu, NULL);
-
-		goto CLEANUP;
-	}
-
-	if (check_current_track(tu))
-	{
-		update_lyrics_window(tu, _("\nLooking for lyrics..."));
-		vfs_async_file_get_contents(uri, (VFSConsumer) get_lyrics_step_3, tu);
-	}
-	else
-	{
-		g_free(uri);
-		goto CLEANUP;
-	}
-
-	g_free(buf);
+	update_lyrics_window(state.title, state.artist, lyrics);
 
+	free(lyrics);
 	return TRUE;
-
-CLEANUP:
-	g_free(buf);
-	tuple_unref (tu);
-	return FALSE;
 }
 
-void
-get_lyrics_step_1(const Tuple *tu)
+static bool_t get_lyrics_step_2(void *buf, int64_t len, void *requri)
 {
-	gchar *uri;
-	gchar *artist, *title;
+	if (strcmp(state.uri, requri))
+	{
+		free(buf);
+		str_unref(requri);
+		return FALSE;
+	}
+	str_unref(requri);
 
-	gchar * artist0 = tuple_get_str (tu, FIELD_ARTIST, NULL);
-	gchar * title0 = tuple_get_str (tu, FIELD_TITLE, NULL);
-	artist = lyricwiki_url_encode (artist0);
-	title = lyricwiki_url_encode (title0);
-	str_unref (artist0);
-	str_unref (title0);
+	if(!len)
+	{
+		SPRINTF(error, _("Unable to fetch %s"), state.uri);
+		update_lyrics_window(_("Error"), NULL, error);
+		free(buf);
+		return FALSE;
+	}
 
-	uri = g_strdup_printf("http://lyrics.wikia.com/api.php?action=lyrics&artist=%s&song=%s&fmt=xml", artist, title);
+	char *uri = scrape_uri_from_lyricwiki_search_result(buf, len);
 
-	g_free(artist);
-	g_free(title);
+	if(!uri)
+	{
+		SPRINTF(error, _("Unable to parse %s"), state.uri);
+		update_lyrics_window(_("Error"), NULL, error);
+		free(buf);
+		return FALSE;
+	}
 
-	update_lyrics_window(tu, _("\nConnecting to lyrics.wikia.com..."));
-	vfs_async_file_get_contents(uri, (VFSConsumer) get_lyrics_step_2, (Tuple *) tu);
+	str_unref(state.uri);
+	state.uri = uri;
 
-	g_free(uri);
+	update_lyrics_window(state.title, state.artist, _("Looking for lyrics ..."));
+	vfs_async_file_get_contents(uri, get_lyrics_step_3, str_ref(state.uri));
+
+	free(buf);
+	return TRUE;
 }
 
-GtkWidget *scrollview, *vbox;
-GtkWidget *textview;
-GtkTextBuffer *textbuffer;
+static void get_lyrics_step_1(void)
+{
+	if(!state.artist || !state.title)
+	{
+		update_lyrics_window(_("Error"), NULL, _("Missing song metadata"));
+		return;
+	}
 
-GtkWidget *
-build_widget(void)
+	char title_buf[strlen(state.title) * 3 + 1];
+	char artist_buf[strlen(state.artist) * 3 + 1];
+	str_encode_percent(state.title, -1, title_buf);
+	str_encode_percent(state.artist, -1, artist_buf);
+
+	str_unref(state.uri);
+	state.uri = str_printf("http://lyrics.wikia.com/api.php?action=lyrics&"
+	 "artist=%s&song=%s&fmt=xml", artist_buf, title_buf);
+
+	update_lyrics_window(state.title, state.artist, _("Connecting to lyrics.wikia.com ..."));
+	vfs_async_file_get_contents(state.uri, get_lyrics_step_2, str_ref(state.uri));
+}
+
+static GtkWidget *scrollview, *vbox;
+static GtkWidget *textview;
+static GtkTextBuffer *textbuffer;
+
+static GtkWidget *build_widget(void)
 {
 	textview = gtk_text_view_new();
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
@@ -325,7 +302,7 @@ build_widget(void)
 	scrollview = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollview), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollview), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	vbox = gtk_vbox_new(FALSE, 10);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
 
 	gtk_container_add(GTK_CONTAINER(scrollview), textview);
 
@@ -343,11 +320,9 @@ build_widget(void)
 	return vbox;
 }
 
-void
-update_lyrics_window(const Tuple *tu, const gchar *lyrics)
+static void update_lyrics_window(const char *title, const char *artist, const char *lyrics)
 {
 	GtkTextIter iter;
-	const gchar *real_lyrics;
 
 	if (textbuffer == NULL)
 		return;
@@ -356,49 +331,42 @@ update_lyrics_window(const Tuple *tu, const gchar *lyrics)
 
 	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(textbuffer), &iter);
 
-	gchar * title = tuple_get_str (tu, FIELD_TITLE, NULL);
-	gchar * artist = tuple_get_str (tu, FIELD_ARTIST, NULL);
-
-	if (! title)
-		title = tuple_get_str (tu, FIELD_FILE_NAME, NULL);
-
 	gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(textbuffer), &iter,
-			title, strlen(title), "weight_bold", "size_x_large", NULL);
-
-	gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, "\n", 1);
+			title, -1, "weight_bold", "size_x_large", NULL);
 
 	if (artist != NULL)
 	{
+		gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, "\n", -1);
 		gtk_text_buffer_insert_with_tags_by_name(GTK_TEXT_BUFFER(textbuffer),
-				&iter, artist, strlen(artist), "style_italic", NULL);
-
-		gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, "\n", 1);
+				&iter, artist, -1, "style_italic", NULL);
 	}
 
-	real_lyrics = lyrics != NULL ? lyrics : _("\nNo lyrics were found.");
-
-	gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, real_lyrics, strlen(real_lyrics));
+	gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, "\n\n", -1);
+	gtk_text_buffer_insert(GTK_TEXT_BUFFER(textbuffer), &iter, lyrics, -1);
 
 	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(textbuffer), &iter);
 	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview), &iter, 0, TRUE, 0, 0);
-
-	str_unref (title);
-	str_unref (artist);
 }
 
-void
-lyricwiki_playback_began(void)
+static void lyricwiki_playback_began(void)
 {
-	gint playlist, pos;
-
 	if (!aud_drct_get_playing())
 		return;
 
-	playlist = aud_playlist_get_playing();
-	pos = aud_playlist_get_position(playlist);
-	Tuple * tu = aud_playlist_entry_get_tuple (playlist, pos, FALSE);
+	/* FIXME: cancel previous VFS requests (not possible with current API) */
+	str_unref(state.filename);
+	str_unref(state.title);
+	str_unref(state.artist);
+	str_unref(state.uri);
 
-	get_lyrics_step_1(tu);
+	int playlist = aud_playlist_get_playing();
+	int pos = aud_playlist_get_position(playlist);
+
+	state.filename = aud_playlist_entry_get_filename(playlist, pos);
+	aud_playlist_entry_describe(playlist, pos, &state.title, &state.artist, NULL, FALSE);
+	state.uri = NULL;
+
+	get_lyrics_step_1();
 }
 
 static gboolean init (void)
@@ -412,9 +380,17 @@ static gboolean init (void)
 	return TRUE;
 }
 
-static void
-cleanup(void)
+static void cleanup(void)
 {
+	str_unref(state.filename);
+	str_unref(state.title);
+	str_unref(state.artist);
+	str_unref(state.uri);
+	state.filename = NULL;
+	state.title = NULL;
+	state.artist = NULL;
+	state.uri = NULL;
+
 	hook_dissociate("title change", (HookFunction) lyricwiki_playback_began);
 	hook_dissociate("playback ready", (HookFunction) lyricwiki_playback_began);
 
@@ -423,8 +399,7 @@ cleanup(void)
 	textbuffer = NULL;
 }
 
-static gpointer
-get_widget(void)
+static void *get_widget(void)
 {
 	if (! vbox)
 		build_widget ();
@@ -433,7 +408,8 @@ get_widget(void)
 
 AUD_GENERAL_PLUGIN
 (
-	.name = "LyricWiki",
+	.name = N_("LyricWiki Plugin"),
+	.domain = PACKAGE,
 	.init = init,
 	.cleanup = cleanup,
 	.get_widget = get_widget,
